@@ -19,15 +19,15 @@ const int init_seq_length = 128;
 // C++ interface
 
 template <typename T>
-size_t get_workspace_size(int maxBatchSize,
-                          int seq_len,
-                          int hidden_size,
-                          int intermediate_size,
-                          int heads,
-                          bool training,
-                          bool gelu_checkpoint)
+int get_workspace_size(int maxBatchSize,
+                       int seq_len,
+                       int hidden_size,
+                       int intermediate_size,
+                       int heads,
+                       bool training,
+                       bool gelu_checkpoint)
 {
-    size_t workSpacesize = 4 * (size_t(maxBatchSize) * seq_len * hidden_size);
+    int workSpacesize = 4 * (size_t(maxBatchSize) * seq_len * hidden_size);
     if (training) {
         workSpacesize += 2 * (size_t(maxBatchSize) * seq_len * hidden_size);
         workSpacesize += ((std::max)((size_t(maxBatchSize) * seq_len * intermediate_size),
