@@ -10,6 +10,8 @@ from .stochastic_transformer import StochasticTransformerBuilder
 from .utils import UtilsBuilder
 from .async_io import AsyncIOBuilder
 from .builder import get_default_compute_capatabilities
+from .transformer_inference import InferenceBuilder
+from .quantizer import QuantizerBuilder
 
 # TODO: infer this list instead of hard coded
 # List of all available ops
@@ -20,7 +22,9 @@ __op_builders__ = [
     SparseAttnBuilder(),
     TransformerBuilder(),
     StochasticTransformerBuilder(),
+    AsyncIOBuilder(),
+    InferenceBuilder(),
     UtilsBuilder(),
-    AsyncIOBuilder()
+    QuantizerBuilder()
 ]
 ALL_OPS = {op.name: op for op in __op_builders__}
