@@ -1,4 +1,6 @@
-import torch
+"""
+Copyright 2020 The Microsoft DeepSpeed Team
+"""
 from .builder import CUDAOpBuilder
 
 
@@ -19,7 +21,11 @@ class FusedAdamBuilder(CUDAOpBuilder):
         return ['csrc/includes']
 
     def cxx_args(self):
-        return ['-O3'] + self.version_dependent_macros()
+        args = super().cxx_args()
+        return args + self.version_dependent_macros()
 
     def nvcc_args(self):
-        return ['-lineinfo', '-O3', '--use_fast_math'] + self.version_dependent_macros()
+        return ['-lineinfo',
+                '-O3',
+                '--use_fast_math'
+                ] + self.version_dependent_macros() + self.compute_capability_args()
